@@ -23,8 +23,6 @@
 #define NETDATA_CHART_PRIO_SYSTEM_IO                    150
 #define NETDATA_CHART_PRIO_SYSTEM_PGPGIO                151
 #define NETDATA_CHART_PRIO_SYSTEM_RAM                   200
-#define NETDATA_CHART_PRIO_SYSTEM_SWAP                  201
-#define NETDATA_CHART_PRIO_SYSTEM_SWAPIO                250
 #define NETDATA_CHART_PRIO_SYSTEM_NET                   500
 #define NETDATA_CHART_PRIO_SYSTEM_IPV4                  500 // freebsd only
 #define NETDATA_CHART_PRIO_SYSTEM_IP                    501
@@ -41,6 +39,7 @@
 #define NETDATA_CHART_PRIO_SYSTEM_INTERRUPTS           1000
 #define NETDATA_CHART_PRIO_SYSTEM_DEV_INTR             1000 // freebsd only
 #define NETDATA_CHART_PRIO_SYSTEM_SOFT_INTR            1100 // freebsd only
+#define NETDATA_CHART_PRIO_SYSTEM_FILES_NR             1000
 #define NETDATA_CHART_PRIO_SYSTEM_ENTROPY              1000
 #define NETDATA_CHART_PRIO_SYSTEM_UPTIME               1000
 #define NETDATA_CHART_PRIO_CLOCK_SYNC_STATE            1100
@@ -76,13 +75,40 @@
 #define NETDATA_CHART_PRIO_MEM_SYSTEM_AVAILABLE       1010
 #define NETDATA_CHART_PRIO_MEM_SYSTEM_OOM_KILL        1020
 #define NETDATA_CHART_PRIO_MEM_SYSTEM_COMMITTED       1030
-#define NETDATA_CHART_PRIO_MEM_SYSTEM_PGFAULTS        1040
+#define NETDATA_CHART_PRIO_MEM_SWAP                   1035
+#define NETDATA_CHART_PRIO_MEM_SWAP_CALLS             1037
+#define NETDATA_CHART_PRIO_MEM_SWAPIO                 1038
+#define NETDATA_CHART_PRIO_MEM_ZSWAP                  1036
+#define NETDATA_CHART_PRIO_MEM_ZSWAPIO                1037
+#define NETDATA_CHART_PRIO_MEM_ZSWAP_COMPRESS_RATIO   1038
+#define NETDATA_CHART_PRIO_MEM_ZSWAP_POOL_TOT_SIZE    1039
+#define NETDATA_CHART_PRIO_MEM_ZSWAP_STORED_PAGE      1040
+#define NETDATA_CHART_PRIO_MEM_ZSWAP_REJECTS          1041
+#define NETDATA_CHART_PRIO_MEM_ZSWAP_POOL_LIM_HIT     1042
+#define NETDATA_CHART_PRIO_MEM_ZSWAP_WRT_BACK_PAGES   1043
+#define NETDATA_CHART_PRIO_MEM_ZSWAP_SAME_FILL_PAGE   1044
+#define NETDATA_CHART_PRIO_MEM_ZSWAP_DUPP_ENTRY       1045
+#define NETDATA_CHART_PRIO_MEM_SYSTEM_PGFAULTS        1050
 #define NETDATA_CHART_PRIO_MEM_KERNEL                 1100
 #define NETDATA_CHART_PRIO_MEM_SLAB                   1200
+#define NETDATA_CHART_PRIO_MEM_RECLAIMING             1210
+#define NETDATA_CHART_PRIO_MEM_HIGH_LOW               1211
+#define NETDATA_CHART_PRIO_MEM_CMA                    1212
 #define NETDATA_CHART_PRIO_MEM_HUGEPAGES              1250
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_DETAILS      1251
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_FAULTS       1252
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_FILE         1253
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_ZERO         1254
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_KHUGEPAGED   1255
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_SPLITS       1256
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_SWAPOUT      1257
+#define NETDATA_CHART_PRIO_MEM_HUGEPAGES_COMPACT      1258
+#define NETDATA_CHART_PRIO_MEM_DIRECTMAP              1260
 #define NETDATA_CHART_PRIO_MEM_KSM                    1300
 #define NETDATA_CHART_PRIO_MEM_KSM_SAVINGS            1301
 #define NETDATA_CHART_PRIO_MEM_KSM_RATIOS             1302
+#define NETDATA_CHART_PRIO_MEM_KSM_COW                1303
+#define NETDATA_CHART_PRIO_MEM_BALLOON                1350
 #define NETDATA_CHART_PRIO_MEM_NUMA                   1400
 #define NETDATA_CHART_PRIO_MEM_NUMA_NODES             1410
 #define NETDATA_CHART_PRIO_MEM_PAGEFRAG               1450
@@ -93,6 +119,7 @@
 #define NETDATA_CHART_PRIO_MEM_ZRAM_SAVINGS           1601
 #define NETDATA_CHART_PRIO_MEM_ZRAM_RATIO             1602
 #define NETDATA_CHART_PRIO_MEM_ZRAM_EFFICIENCY        1603
+#define NETDATA_CHART_PRIO_MEM_FRAGMENTATION          1700
 
 // Disks
 
@@ -182,6 +209,10 @@
 #define NETDATA_CHART_PRIO_BTRFS_DATA                 2401
 #define NETDATA_CHART_PRIO_BTRFS_METADATA             2402
 #define NETDATA_CHART_PRIO_BTRFS_SYSTEM               2403
+#define NETDATA_CHART_PRIO_BTRFS_COMMITS              2404
+#define NETDATA_CHART_PRIO_BTRFS_COMMITS_PERC_TIME    2405
+#define NETDATA_CHART_PRIO_BTRFS_COMMIT_TIMINGS       2406
+#define NETDATA_CHART_PRIO_BTRFS_ERRORS               2407
 
 // ZFS
 
@@ -342,10 +373,13 @@
 #define NETDATA_CHART_PRIO_POWER_SUPPLY_ENERGY        9502
 #define NETDATA_CHART_PRIO_POWER_SUPPLY_VOLTAGE       9503
 
+// Linux powercap
+
+#define NETDATA_CHART_PRIO_POWERCAP                   9600
 
 // Wireless
 
-#define NETDATA_CHART_PRIO_WIRELESS_IFACE	     7110
+#define NETDATA_CHART_PRIO_WIRELESS_IFACE	          7110
 
 // CGROUPS
 
@@ -356,12 +390,39 @@
 
 #define NETDATA_CHART_PRIO_STATSD_PRIVATE            90000 // many charts
 
-// INTERNAL NETDATA INFO
+// PCI
 
-#define NETDATA_CHART_PRIO_CHECKS                    99999
+#define NETDATA_CHART_PRIO_PCI_AER                  100000
 
-#define NETDATA_CHART_PRIO_NETDATA_TIMEX            132030
-#define NETDATA_CHART_PRIO_NETDATA_TC_TIME          1000100
+// AMD GPUs
+
+#define NETDATA_CHART_PRIO_DRM_AMDGPU                 39000
+
+// NETDATA ML CHARTS
+
+// [ml] charts
+#define ML_CHART_PRIO_DIMENSIONS                      39181
+#define ML_CHART_PRIO_ANOMALY_RATE                    39182
+#define ML_CHART_PRIO_DETECTOR_EVENTS                 39183
+
+// [netdata.ml] charts
+#define NETDATA_ML_CHART_RUNNING                      890001
+#define NETDATA_ML_CHART_PRIO_MACHINE_LEARNING_STATUS 890002
+#define NETDATA_ML_CHART_PRIO_METRIC_TYPES            890003
+#define NETDATA_ML_CHART_PRIO_TRAINING_STATUS         890004
+
+#define NETDATA_ML_CHART_PRIO_PREDICTION_USAGE        890005
+#define NETDATA_ML_CHART_PRIO_TRAINING_USAGE          890006
+
+#define NETDATA_ML_CHART_PRIO_QUEUE_STATS             890007
+#define NETDATA_ML_CHART_PRIO_TRAINING_TIME_STATS     890008
+#define NETDATA_ML_CHART_PRIO_TRAINING_RESULTS        890009
+
+#define NETDATA_ML_CHART_FAMILY "machine learning"
+#define NETDATA_ML_PLUGIN "ml.plugin"
+#define NETDATA_ML_MODULE_TRAINING "training"
+#define NETDATA_ML_MODULE_DETECTION "detection"
+#define NETDATA_ML_MODULE_PREDICTION "prediction"
 
 
 #endif //NETDATA_ALL_H
