@@ -3,10 +3,10 @@
 #include "../proc.plugin/plugin_proc.h"
 
 #define PLUGIN_DISKSPACE_NAME "diskspace.plugin"
-#define THREAD_DISKSPACE_SLOW_NAME "PLUGIN[diskspace slow]"
 
 #define DEFAULT_EXCLUDED_PATHS "/proc/* /sys/* /var/run/user/* /run/user/* /snap/* /var/lib/docker/*"
 #define DEFAULT_EXCLUDED_FILESYSTEMS "*gvfs *gluster* *s3fs *ipfs *davfs2 *httpfs *sshfs *gdfs *moosefs fusectl autofs"
+#define DEFAULT_EXCLUDED_FILESYSTEMS_INODES "msdosfs msdos vfat overlayfs aufs* *unionfs"
 #define CONFIG_SECTION_DISKSPACE "plugin:proc:diskspace"
 
 #define MAX_STAT_USEC 10000LU
@@ -182,7 +182,7 @@ static void calculate_values_and_show_charts(
 
 #ifdef NETDATA_INTERNAL_CHECKS
     if(unlikely(btotal != bavail + breserved_root + bused))
-        error("DISKSPACE: disk block statistics for '%s' (disk '%s') do not sum up: total = %llu, available = %llu, reserved = %llu, used = %llu", mi->mount_point, disk, (unsigned long long)btotal, (unsigned long long)bavail, (unsigned long long)breserved_root, (unsigned long long)bused);
+        collector_error("DISKSPACE: disk block statistics for '%s' (disk '%s') do not sum up: total = %llu, available = %llu, reserved = %llu, used = %llu", mi->mount_point, disk, (unsigned long long)btotal, (unsigned long long)bavail, (unsigned long long)breserved_root, (unsigned long long)bused);
 #endif
 
     // --------------------------------------------------------------------------
@@ -201,7 +201,7 @@ static void calculate_values_and_show_charts(
 
 #ifdef NETDATA_INTERNAL_CHECKS
     if(unlikely(btotal != bavail + breserved_root + bused))
-        error("DISKSPACE: disk inode statistics for '%s' (disk '%s') do not sum up: total = %llu, available = %llu, reserved = %llu, used = %llu", mi->mount_point, disk, (unsigned long long)ftotal, (unsigned long long)favail, (unsigned long long)freserved_root, (unsigned long long)fused);
+        collector_error("DISKSPACE: disk inode statistics for '%s' (disk '%s') do not sum up: total = %llu, available = %llu, reserved = %llu, used = %llu", mi->mount_point, disk, (unsigned long long)ftotal, (unsigned long long)favail, (unsigned long long)freserved_root, (unsigned long long)fused);
 #endif
 
     int rendered = 0;
@@ -295,6 +295,7 @@ static inline void do_disk_space_stats(struct mountinfo *mi, int update_every) {
 
     static SIMPLE_PATTERN *excluded_mountpoints = NULL;
     static SIMPLE_PATTERN *excluded_filesystems = NULL;
+    static SIMPLE_PATTERN *excluded_filesystems_inodes = NULL;
 
     usec_t slow_timeout = MAX_STAT_USEC * update_every;
 
@@ -309,18 +310,24 @@ static inline void do_disk_space_stats(struct mountinfo *mi, int update_every) {
         }
 
         excluded_mountpoints = simple_pattern_create(
-                config_get(CONFIG_SECTION_DISKSPACE, "exclude space metrics on paths", DEFAULT_EXCLUDED_PATHS)
-                , NULL
-                , mode
-        );
+            config_get(CONFIG_SECTION_DISKSPACE, "exclude space metrics on paths", DEFAULT_EXCLUDED_PATHS),
+            NULL,
+            mode,
+            true);
 
         excluded_filesystems = simple_pattern_create(
-                config_get(CONFIG_SECTION_DISKSPACE, "exclude space metrics on filesystems", DEFAULT_EXCLUDED_FILESYSTEMS)
-                , NULL
-                , SIMPLE_PATTERN_EXACT
-        );
+            config_get(CONFIG_SECTION_DISKSPACE, "exclude space metrics on filesystems", DEFAULT_EXCLUDED_FILESYSTEMS),
+            NULL,
+            SIMPLE_PATTERN_EXACT,
+            true);
 
-        dict_mountpoints = dictionary_create(DICT_OPTION_NONE);
+        excluded_filesystems_inodes = simple_pattern_create(
+            config_get(CONFIG_SECTION_DISKSPACE, "exclude inode metrics on filesystems", DEFAULT_EXCLUDED_FILESYSTEMS_INODES),
+            NULL,
+            SIMPLE_PATTERN_EXACT,
+            true);
+
+        dict_mountpoints = dictionary_create_advanced(DICT_OPTION_NONE, &dictionary_stats_category_collectors, 0);
     }
 
     struct mount_point_metadata *m = dictionary_get(dict_mountpoints, mi->mount_point);
@@ -341,6 +348,9 @@ static inline void do_disk_space_stats(struct mountinfo *mi, int update_every) {
             def_space = CONFIG_BOOLEAN_NO;
             def_inodes = CONFIG_BOOLEAN_NO;
         }
+        if (unlikely(simple_pattern_matches(excluded_filesystems_inodes, mi->filesystem))) {
+            def_inodes = CONFIG_BOOLEAN_NO;
+        }
 
         // check if the mount point is a directory #2407
         // but only when it is enabled by default #4491
@@ -349,23 +359,23 @@ static inline void do_disk_space_stats(struct mountinfo *mi, int update_every) {
             struct stat bs;
 
             if(stat(mi->mount_point, &bs) == -1) {
-                error("DISKSPACE: Cannot stat() mount point '%s' (disk '%s', filesystem '%s', root '%s')."
-                      , mi->mount_point
-                      , disk
-                      , mi->filesystem?mi->filesystem:""
-                      , mi->root?mi->root:""
-                );
+                collector_error("DISKSPACE: Cannot stat() mount point '%s' (disk '%s', filesystem '%s', root '%s')."
+                               , mi->mount_point
+                               , disk
+                               , mi->filesystem?mi->filesystem:""
+                               , mi->root?mi->root:""
+                               );
                 def_space = CONFIG_BOOLEAN_NO;
                 def_inodes = CONFIG_BOOLEAN_NO;
             }
             else {
                 if((bs.st_mode & S_IFMT) != S_IFDIR) {
-                    error("DISKSPACE: Mount point '%s' (disk '%s', filesystem '%s', root '%s') is not a directory."
-                          , mi->mount_point
-                          , disk
-                          , mi->filesystem?mi->filesystem:""
-                          , mi->root?mi->root:""
-                    );
+                    collector_error("DISKSPACE: Mount point '%s' (disk '%s', filesystem '%s', root '%s') is not a directory."
+                                   , mi->mount_point
+                                   , disk
+                                   , mi->filesystem?mi->filesystem:""
+                                   , mi->root?mi->root:""
+                                   );
                     def_space = CONFIG_BOOLEAN_NO;
                     def_inodes = CONFIG_BOOLEAN_NO;
                 }
@@ -431,12 +441,12 @@ static inline void do_disk_space_stats(struct mountinfo *mi, int update_every) {
 
     if (statvfs(mi->mount_point, &buff_statvfs) < 0) {
         if(!m->shown_error) {
-            error("DISKSPACE: failed to statvfs() mount point '%s' (disk '%s', filesystem '%s', root '%s')"
-                  , mi->mount_point
-                  , disk
-                  , mi->filesystem?mi->filesystem:""
-                  , mi->root?mi->root:""
-            );
+            collector_error("DISKSPACE: failed to statvfs() mount point '%s' (disk '%s', filesystem '%s', root '%s')"
+                            , mi->mount_point
+                            , disk
+                            , mi->filesystem?mi->filesystem:""
+                            , mi->root?mi->root:""
+                            );
             m->shown_error = 1;
         }
         return;
@@ -464,12 +474,12 @@ static inline void do_slow_disk_space_stats(struct basic_mountinfo *mi, int upda
     struct statvfs buff_statvfs;
     if (statvfs(mi->mount_point, &buff_statvfs) < 0) {
         if(!m->shown_error) {
-            error("DISKSPACE: failed to statvfs() mount point '%s' (disk '%s', filesystem '%s', root '%s')"
-                  , mi->mount_point
-                  , mi->persistent_id
-                  , mi->filesystem?mi->filesystem:""
-                  , mi->root?mi->root:""
-            );
+            collector_error("DISKSPACE: failed to statvfs() mount point '%s' (disk '%s', filesystem '%s', root '%s')"
+                            , mi->mount_point
+                            , mi->persistent_id
+                            , mi->filesystem?mi->filesystem:""
+                            , mi->root?mi->root:""
+                           );
             m->shown_error = 1;
         }
         return;
@@ -483,7 +493,7 @@ static void diskspace_slow_worker_cleanup(void *ptr)
 {
     UNUSED(ptr);
 
-    info("cleaning up...");
+    collector_info("cleaning up...");
 
     worker_unregister();
 }
@@ -515,7 +525,7 @@ void *diskspace_slow_worker(void *ptr)
     heartbeat_t hb;
     heartbeat_init(&hb);
 
-    while(!netdata_exit) {
+    while(service_running(SERVICE_COLLECTORS)) {
         worker_is_idle();
         heartbeat_next(&hb, USEC_PER_SEC);
 
@@ -530,7 +540,7 @@ void *diskspace_slow_worker(void *ptr)
         if (!dict_mountpoints)
             continue;
 
-        if(unlikely(netdata_exit)) break;
+        if(unlikely(!service_running(SERVICE_COLLECTORS))) break;
 
         // --------------------------------------------------------------------------
         // disk space metrics
@@ -547,10 +557,10 @@ void *diskspace_slow_worker(void *ptr)
         for(bmi = slow_mountinfo_root; bmi; bmi = bmi->next) {
             do_slow_disk_space_stats(bmi, slow_update_every);
             
-            if(unlikely(netdata_exit)) break;
+            if(unlikely(!service_running(SERVICE_COLLECTORS))) break;
         }
 
-        if(unlikely(netdata_exit)) break;
+        if(unlikely(!service_running(SERVICE_COLLECTORS))) break;
 
         worker_is_busy(WORKER_JOB_SLOW_CLEANUP);
 
@@ -584,7 +594,7 @@ static void diskspace_main_cleanup(void *ptr) {
     struct netdata_static_thread *static_thread = (struct netdata_static_thread *)ptr;
     static_thread->enabled = NETDATA_MAIN_THREAD_EXITING;
 
-    info("cleaning up...");
+    collector_info("cleaning up...");
 
     if (diskspace_slow_thread) {
         netdata_thread_join(*diskspace_slow_thread, NULL);
@@ -632,7 +642,7 @@ void *diskspace_main(void *ptr) {
 
     netdata_thread_create(
         diskspace_slow_thread,
-        THREAD_DISKSPACE_SLOW_NAME,
+        "P[diskspace slow]",
         NETDATA_THREAD_OPTION_JOINABLE,
         diskspace_slow_worker,
         &slow_worker_data);
@@ -640,11 +650,11 @@ void *diskspace_main(void *ptr) {
     usec_t step = update_every * USEC_PER_SEC;
     heartbeat_t hb;
     heartbeat_init(&hb);
-    while(!netdata_exit) {
+    while(service_running(SERVICE_COLLECTORS)) {
         worker_is_idle();
         /* usec_t hb_dt = */ heartbeat_next(&hb, step);
 
-        if(unlikely(netdata_exit)) break;
+        if(unlikely(!service_running(SERVICE_COLLECTORS))) break;
 
         // --------------------------------------------------------------------------
         // this is smart enough not to reload it every time
@@ -671,11 +681,11 @@ void *diskspace_main(void *ptr) {
 
             worker_is_busy(WORKER_JOB_MOUNTPOINT);
             do_disk_space_stats(mi, update_every);
-            if(unlikely(netdata_exit)) break;
+            if(unlikely(!service_running(SERVICE_COLLECTORS))) break;
         }
         netdata_mutex_unlock(&slow_mountinfo_mutex);
 
-        if(unlikely(netdata_exit)) break;
+        if(unlikely(!service_running(SERVICE_COLLECTORS))) break;
 
         if(dict_mountpoints) {
             worker_is_busy(WORKER_JOB_CLEANUP);
