@@ -8,7 +8,7 @@
 #define PLUGIN_PROC_CONFIG_NAME "proc"
 #define PLUGIN_PROC_NAME PLUGIN_PROC_CONFIG_NAME ".plugin"
 
-#define THREAD_NETDEV_NAME "PLUGIN[proc netdev]"
+#define THREAD_NETDEV_NAME "P[proc netdev]"
 void *netdev_main(void *ptr);
 
 int do_proc_net_wireless(int update_every, usec_t dt);
@@ -22,6 +22,7 @@ int do_proc_meminfo(int update_every, usec_t dt);
 int do_proc_vmstat(int update_every, usec_t dt);
 int do_proc_net_rpc_nfs(int update_every, usec_t dt);
 int do_proc_net_rpc_nfsd(int update_every, usec_t dt);
+int do_proc_sys_fs_file_nr(int update_every, usec_t dt);
 int do_proc_sys_kernel_random_entropy_avail(int update_every, usec_t dt);
 int do_proc_interrupts(int update_every, usec_t dt);
 int do_proc_softirqs(int update_every, usec_t dt);
@@ -33,6 +34,7 @@ int do_proc_net_stat_synproxy(int update_every, usec_t dt);
 int do_proc_net_softnet_stat(int update_every, usec_t dt);
 int do_proc_uptime(int update_every, usec_t dt);
 int do_proc_sys_devices_system_edac_mc(int update_every, usec_t dt);
+int do_proc_sys_devices_pci_aer(int update_every, usec_t dt);
 int do_proc_sys_devices_system_node(int update_every, usec_t dt);
 int do_proc_spl_kstat_zfs_arcstats(int update_every, usec_t dt);
 int do_proc_spl_kstat_zfs_pool_state(int update_every, usec_t dt);
@@ -44,10 +46,12 @@ int do_ipc(int update_every, usec_t dt);
 int do_sys_class_power_supply(int update_every, usec_t dt);
 int do_proc_pagetypeinfo(int update_every, usec_t dt);
 int do_sys_class_infiniband(int update_every, usec_t dt);
+int do_sys_class_drm(int update_every, usec_t dt);
 int get_numa_node_count(void);
 
 // metrics that need to be shared among data collectors
 extern unsigned long long zfs_arcstats_shrinkable_cache_size_bytes;
+extern bool inside_lxc_container;
 
 // netdev renames
 void netdev_rename_device_add(
