@@ -3,7 +3,7 @@
 #include "sqlite_functions.h"
 #include "sqlite_db_migration.h"
 
-#define DB_METADATA_VERSION 7
+#define DB_METADATA_VERSION 11
 
 const char *database_config[] = {
     "CREATE TABLE IF NOT EXISTS host(host_id BLOB PRIMARY KEY, hostname TEXT NOT NULL, "
@@ -22,9 +22,8 @@ const char *database_config[] = {
     "multiplier int, divisor int , algorithm int, options text);",
 
     "CREATE TABLE IF NOT EXISTS metadata_migration(filename text, file_size, date_created int);",
-    "CREATE INDEX IF NOT EXISTS ind_d1 on dimension (chart_id, id, name);",
-    "CREATE INDEX IF NOT EXISTS ind_c1 on chart (host_id, id, type, name);",
-    "CREATE INDEX IF NOT EXISTS ind_c2 on chart (host_id, context);",
+    "CREATE INDEX IF NOT EXISTS ind_d2 on dimension (chart_id);",
+    "CREATE INDEX IF NOT EXISTS ind_c3 on chart (host_id);",
     "CREATE TABLE IF NOT EXISTS chart_label(chart_id blob, source_type int, label_key text, "
     "label_value text, date_created int, PRIMARY KEY (chart_id, label_key));",
     "CREATE TABLE IF NOT EXISTS node_instance (host_id blob PRIMARY KEY, claim_id, node_id, date_created);",
@@ -33,7 +32,7 @@ const char *database_config[] = {
     "every text, units text, calc text, families text, plugin text, module text, charts text, green text, "
     "red text, warn text, crit text, exec text, to_key text, info text, delay text, options text, "
     "repeat text, host_labels text, p_db_lookup_dimensions text, p_db_lookup_method text, p_db_lookup_options int, "
-    "p_db_lookup_after int, p_db_lookup_before int, p_update_every int);",
+    "p_db_lookup_after int, p_db_lookup_before int, p_update_every int, source text, chart_labels text);",
 
     "CREATE TABLE IF NOT EXISTS host_info(host_id blob, system_key text NOT NULL, system_value text NOT NULL, "
     "date_created INT, PRIMARY KEY(host_id, system_key));",
@@ -44,17 +43,37 @@ const char *database_config[] = {
     "CREATE TRIGGER IF NOT EXISTS ins_host AFTER INSERT ON host BEGIN INSERT INTO node_instance (host_id, date_created)"
       " SELECT new.host_id, unixepoch() WHERE new.host_id NOT IN (SELECT host_id FROM node_instance); END;",
 
+    "CREATE TABLE IF NOT EXISTS health_log (health_log_id INTEGER PRIMARY KEY, host_id blob, alarm_id int, "
+    "config_hash_id blob, name text, chart text, family text, recipient text, units text, exec text, "
+    "chart_context text, last_transition_id blob, chart_name text, UNIQUE (host_id, alarm_id)) ;",
+
+    "CREATE INDEX IF NOT EXISTS health_log_ind_1 ON health_log (host_id);",
+
+    "CREATE TABLE IF NOT EXISTS health_log_detail (health_log_id int, unique_id int, alarm_id int, alarm_event_id int, "
+    "updated_by_id int, updates_id int, when_key int, duration int, non_clear_duration int, "
+    "flags int, exec_run_timestamp int, delay_up_to_timestamp int, "
+    "info text, exec_code int, new_status real, old_status real, delay int, "
+    "new_value double, old_value double, last_repeat int, transition_id blob, global_id int);",
+
+    "CREATE INDEX IF NOT EXISTS health_log_d_ind_1 ON health_log_detail (unique_id);",
+    "CREATE INDEX IF NOT EXISTS health_log_d_ind_2 ON health_log_detail (global_id);",
+    "CREATE INDEX IF NOT EXISTS health_log_d_ind_3 ON health_log_detail (transition_id);",
+    "CREATE INDEX IF NOT EXISTS health_log_d_ind_4 ON health_log_detail (health_log_id);",
+
     NULL
 };
 
 const char *database_cleanup[] = {
     "DELETE FROM chart WHERE chart_id NOT IN (SELECT chart_id FROM dimension);",
     "DELETE FROM host WHERE host_id NOT IN (SELECT host_id FROM chart);",
-    "DELETE FROM chart_label WHERE chart_id NOT IN (SELECT chart_id FROM chart);",
     "DELETE FROM node_instance WHERE host_id NOT IN (SELECT host_id FROM host);",
     "DELETE FROM host_info WHERE host_id NOT IN (SELECT host_id FROM host);",
     "DELETE FROM host_label WHERE host_id NOT IN (SELECT host_id FROM host);",
     "DROP TRIGGER IF EXISTS tr_dim_del;",
+    "DROP INDEX IF EXISTS ind_d1;",
+    "DROP INDEX IF EXISTS ind_c1;",
+    "DROP INDEX IF EXISTS ind_c2;",
+    "DROP INDEX IF EXISTS alert_hash_index;",
     NULL
 };
 
@@ -115,7 +134,6 @@ int execute_insert(sqlite3_stmt *res)
             break;
         }
     }
-
     return rc;
 }
 
@@ -128,9 +146,9 @@ static void add_stmt_to_list(sqlite3_stmt *res)
 
     if (unlikely(!res)) {
         if (idx)
-            info("Finilizing %d statements", idx);
+            netdata_log_info("Finilizing %d statements", idx);
         else
-            info("No statements pending to finalize");
+            netdata_log_info("No statements pending to finalize");
         while (idx > 0) {
             int rc;
             rc = sqlite3_finalize(statements[--idx]);
@@ -148,12 +166,18 @@ static void release_statement(void *statement)
 {
     int rc;
 #ifdef NETDATA_DEV_MODE
-    info("Thread %d: Cleaning prepared statement on %p", gettid(), statement);
+    netdata_log_info("Thread %d: Cleaning prepared statement on %p", gettid(), statement);
 #endif
     if (unlikely(rc = sqlite3_finalize((sqlite3_stmt *) statement) != SQLITE_OK))
         error_report("Failed to finalize statement, rc = %d", rc);
 }
 
+void initialize_thread_key_pool(void)
+{
+    for (int i = 0; i < MAX_PREPARED_STATEMENTS; i++)
+        (void)pthread_key_create(&key_pool[i], release_statement);
+}
+
 int prepare_statement(sqlite3 *database, const char *query, sqlite3_stmt **statement)
 {
     static __thread uint32_t keys_used = 0;
@@ -169,7 +193,7 @@ int prepare_statement(sqlite3 *database, const char *query, sqlite3_stmt **state
         if (likely(key)) {
             ret = pthread_setspecific(*key, *statement);
 #ifdef NETDATA_DEV_MODE
-            info("Thread %d: Using key %u on statement %p", gettid(), keys_used, *statement);
+            netdata_log_info("Thread %d: Using key %u on statement %p", gettid(), keys_used, *statement);
 #endif
         }
         if (ret)
@@ -183,7 +207,7 @@ static int check_table_integrity_cb(void *data, int argc, char **argv, char **co
     int *status = data;
     UNUSED(argc);
     UNUSED(column);
-    info("---> %s", argv[0]);
+    netdata_log_info("---> %s", argv[0]);
     *status = (strcmp(argv[0], "ok") != 0);
     return 0;
 }
@@ -196,11 +220,11 @@ static int check_table_integrity(char *table)
     char wstr[255];
 
     if (table) {
-        info("Checking table %s", table);
+        netdata_log_info("Checking table %s", table);
         snprintfz(wstr, 254, "PRAGMA integrity_check(%s);", table);
     }
     else {
-        info("Checking entire database");
+        netdata_log_info("Checking entire database");
         strcpy(wstr,"PRAGMA integrity_check;");
     }
 
@@ -234,9 +258,9 @@ static void rebuild_chart()
 {
     int rc;
     char *err_msg = NULL;
-    info("Rebuilding chart table");
+    netdata_log_info("Rebuilding chart table");
     for (int i = 0; rebuild_chart_commands[i]; i++) {
-        info("Executing %s", rebuild_chart_commands[i]);
+        netdata_log_info("Executing %s", rebuild_chart_commands[i]);
         rc = sqlite3_exec_monitored(db_meta, rebuild_chart_commands[i], 0, 0, &err_msg);
         if (rc != SQLITE_OK) {
             error_report("SQLite error during database setup, rc = %d (%s)", rc, err_msg);
@@ -266,9 +290,9 @@ void rebuild_dimension()
     int rc;
     char *err_msg = NULL;
 
-    info("Rebuilding dimension table");
+    netdata_log_info("Rebuilding dimension table");
     for (int i = 0; rebuild_dimension_commands[i]; i++) {
-        info("Executing %s", rebuild_dimension_commands[i]);
+        netdata_log_info("Executing %s", rebuild_dimension_commands[i]);
         rc = sqlite3_exec_monitored(db_meta, rebuild_dimension_commands[i], 0, 0, &err_msg);
         if (rc != SQLITE_OK) {
             error_report("SQLite error during database setup, rc = %d (%s)", rc, err_msg);
@@ -280,11 +304,11 @@ void rebuild_dimension()
 
 static int attempt_database_fix()
 {
-    info("Closing database and attempting to fix it");
+    netdata_log_info("Closing database and attempting to fix it");
     int rc = sqlite3_close(db_meta);
     if (rc != SQLITE_OK)
         error_report("Failed to close database, rc = %d", rc);
-    info("Attempting to fix database");
+    netdata_log_info("Attempting to fix database");
     db_meta = NULL;
     return sql_init_database(DB_CHECK_FIX_DB | DB_CHECK_CONT, 0);
 }
@@ -294,7 +318,7 @@ int init_database_batch(sqlite3 *database, int rebuild, int init_type, const cha
     int rc;
     char *err_msg = NULL;
     for (int i = 0; batch[i]; i++) {
-        debug(D_METADATALOG, "Executing %s", batch[i]);
+        netdata_log_debug(D_METADATALOG, "Executing %s", batch[i]);
         rc = sqlite3_exec_monitored(database, batch[i], 0, 0, &err_msg);
         if (rc != SQLITE_OK) {
             error_report("SQLite error during database %s, rc = %d (%s)", init_type ? "cleanup" : "setup", rc, err_msg);
@@ -330,6 +354,30 @@ static void sqlite_uuid_parse(sqlite3_context *context, int argc, sqlite3_value
     sqlite3_result_blob(context, &uuid, sizeof(uuid_t), SQLITE_TRANSIENT);
 }
 
+void sqlite_now_usec(sqlite3_context *context, int argc, sqlite3_value **argv)
+{
+    if (argc != 1 ){
+        sqlite3_result_null(context);
+        return ;
+    }
+
+    if (sqlite3_value_int(argv[0]) != 0) {
+        struct timespec req = {.tv_sec = 0, .tv_nsec = 1};
+        nanosleep(&req, NULL);
+    }
+
+    sqlite3_result_int64(context, (sqlite_int64) now_realtime_usec());
+}
+
+void sqlite_uuid_random(sqlite3_context *context, int argc, sqlite3_value **argv)
+{
+    (void)argc;
+    (void)argv;
+
+    uuid_t uuid;
+    uuid_generate_random(uuid);
+    sqlite3_result_blob(context, &uuid, sizeof(uuid_t), SQLITE_TRANSIENT);
+}
 
 /*
  * Initialize the SQLite database
@@ -357,7 +405,7 @@ int sql_init_database(db_check_action_type_t rebuild, int memory)
     if (rebuild & (DB_CHECK_INTEGRITY | DB_CHECK_FIX_DB)) {
         int errors_detected = 0;
         if (!(rebuild & DB_CHECK_CONT))
-            info("Running database check on %s", sqlite_database);
+            netdata_log_info("Running database check on %s", sqlite_database);
 
         if (check_table_integrity("chart")) {
             errors_detected++;
@@ -383,7 +431,7 @@ int sql_init_database(db_check_action_type_t rebuild, int memory)
 
     if (rebuild & DB_CHECK_RECLAIM_SPACE) {
         if (!(rebuild & DB_CHECK_CONT))
-            info("Reclaiming space of %s", sqlite_database);
+            netdata_log_info("Reclaiming space of %s", sqlite_database);
         rc = sqlite3_exec_monitored(db_meta, "VACUUM;", 0, 0, &err_msg);
         if (rc != SQLITE_OK) {
             error_report("Failed to execute VACUUM rc = %d (%s)", rc, err_msg);
@@ -394,11 +442,23 @@ int sql_init_database(db_check_action_type_t rebuild, int memory)
     if (rebuild && !(rebuild & DB_CHECK_CONT))
         return 1;
 
-    info("SQLite database %s initialization", sqlite_database);
+    netdata_log_info("SQLite database %s initialization", sqlite_database);
 
     char buf[1024 + 1] = "";
     const char *list[2] = { buf, NULL };
 
+    rc = sqlite3_create_function(db_meta, "u2h", 1, SQLITE_ANY | SQLITE_DETERMINISTIC, 0, sqlite_uuid_parse, 0, 0);
+    if (unlikely(rc != SQLITE_OK))
+        error_report("Failed to register internal u2h function");
+
+    rc = sqlite3_create_function(db_meta, "now_usec", 1, SQLITE_ANY, 0, sqlite_now_usec, 0, 0);
+    if (unlikely(rc != SQLITE_OK))
+        error_report("Failed to register internal now_usec function");
+
+    rc = sqlite3_create_function(db_meta, "uuid_random", 0, SQLITE_ANY, 0, sqlite_uuid_random, 0, 0);
+    if (unlikely(rc != SQLITE_OK))
+        error_report("Failed to register internal uuid_random function");
+
     int target_version = DB_METADATA_VERSION;
 
     if (likely(!memory))
@@ -444,14 +504,10 @@ int sql_init_database(db_check_action_type_t rebuild, int memory)
     if (init_database_batch(db_meta, rebuild, 0, &database_cleanup[0]))
         return 1;
 
-    info("SQLite database initialization completed");
+    netdata_log_info("SQLite database initialization completed");
 
-    for (int i = 0; i < MAX_PREPARED_STATEMENTS; i++)
-        (void)pthread_key_create(&key_pool[i], release_statement);
+    initialize_thread_key_pool();
 
-    rc = sqlite3_create_function(db_meta, "u2h", 1, SQLITE_ANY | SQLITE_DETERMINISTIC, 0, sqlite_uuid_parse, 0, 0);
-    if (unlikely(rc != SQLITE_OK))
-        error_report("Failed to register internal u2h function");
     return 0;
 }
 
@@ -465,7 +521,7 @@ void sql_close_database(void)
     if (unlikely(!db_meta))
         return;
 
-    info("Closing SQLite database");
+    netdata_log_info("Closing SQLite database");
 
     add_stmt_to_list(NULL);
 
@@ -503,219 +559,15 @@ skip:
         error_report("Failed to finalize statement %s, rc = %d", sql, rc);
     return result;
 }
-
-
-
-//
-// Support for archived charts (TO BE REMOVED)
-//
-#define SELECT_DIMENSION "select d.id, d.name from dimension d where d.chart_id = @chart_uuid;"
-
-static void sql_rrdim2json(sqlite3_stmt *res_dim, uuid_t *chart_uuid, BUFFER *wb, size_t *dimensions_count)
-{
-    int rc;
-
-    rc = sqlite3_bind_blob(res_dim, 1, chart_uuid, sizeof(*chart_uuid), SQLITE_STATIC);
-    if (rc != SQLITE_OK)
-        return;
-
-    int dimensions = 0;
-    buffer_sprintf(wb, "\t\t\t\"dimensions\": {\n");
-
-    while (sqlite3_step_monitored(res_dim) == SQLITE_ROW) {
-        if (dimensions)
-            buffer_strcat(wb, ",\n\t\t\t\t\"");
-        else
-            buffer_strcat(wb, "\t\t\t\t\"");
-        buffer_strcat_jsonescape(wb, (const char *) sqlite3_column_text(res_dim, 0));
-        buffer_strcat(wb, "\": { \"name\": \"");
-        buffer_strcat_jsonescape(wb, (const char *) sqlite3_column_text(res_dim, 1));
-        buffer_strcat(wb, "\" }");
-        dimensions++;
-    }
-    *dimensions_count += dimensions;
-    buffer_sprintf(wb, "\n\t\t\t}");
-}
-
-#define SELECT_CHART "select chart_id, id, name, type, family, context, title, priority, plugin, " \
-    "module, unit, chart_type, update_every from chart " \
-    "where host_id = @host_uuid and chart_id not in (select chart_id from chart_active) order by chart_id asc;"
-
-void sql_rrdset2json(RRDHOST *host, BUFFER *wb)
-{
-    //    time_t first_entry_t = 0; //= rrdset_first_entry_t(st);
-    //   time_t last_entry_t = 0; //rrdset_last_entry_t(st);
-    static char *custom_dashboard_info_js_filename = NULL;
-    int rc;
-
-    sqlite3_stmt *res_chart = NULL;
-    sqlite3_stmt *res_dim = NULL;
-    time_t now = now_realtime_sec();
-
-    rc = sqlite3_prepare_v2(db_meta, SELECT_CHART, -1, &res_chart, 0);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to prepare statement to fetch host archived charts");
-        return;
-    }
-
-    rc = sqlite3_bind_blob(res_chart, 1, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to bind host parameter to fetch archived charts");
-        goto failed;
-    }
-
-    rc = sqlite3_prepare_v2(db_meta, SELECT_DIMENSION, -1, &res_dim, 0);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to prepare statement to fetch chart archived dimensions");
-        goto failed;
-    };
-
-    if(unlikely(!custom_dashboard_info_js_filename))
-        custom_dashboard_info_js_filename = config_get(CONFIG_SECTION_WEB, "custom dashboard_info.js", "");
-
-    buffer_sprintf(wb, "{\n"
-                       "\t\"hostname\": \"%s\""
-                       ",\n\t\"version\": \"%s\""
-                       ",\n\t\"release_channel\": \"%s\""
-                       ",\n\t\"os\": \"%s\""
-                       ",\n\t\"timezone\": \"%s\""
-                       ",\n\t\"update_every\": %d"
-                       ",\n\t\"history\": %ld"
-                       ",\n\t\"memory_mode\": \"%s\""
-                       ",\n\t\"custom_info\": \"%s\""
-                       ",\n\t\"charts\": {"
-        , rrdhost_hostname(host)
-        , rrdhost_program_version(host)
-        , get_release_channel()
-        , rrdhost_os(host)
-        , rrdhost_timezone(host)
-        , host->rrd_update_every
-        , host->rrd_history_entries
-        , rrd_memory_mode_name(host->rrd_memory_mode)
-        , custom_dashboard_info_js_filename
-    );
-
-    size_t c = 0;
-    size_t dimensions = 0;
-
-    while (sqlite3_step_monitored(res_chart) == SQLITE_ROW) {
-        char id[512];
-        sprintf(id, "%s.%s", sqlite3_column_text(res_chart, 3), sqlite3_column_text(res_chart, 1));
-        RRDSET *st = rrdset_find(host, id);
-        if (st && !rrdset_flag_check(st, RRDSET_FLAG_ARCHIVED))
-            continue;
-
-        if (c)
-            buffer_strcat(wb, ",\n\t\t\"");
-        else
-            buffer_strcat(wb, "\n\t\t\"");
-        c++;
-
-        buffer_strcat(wb, id);
-        buffer_strcat(wb, "\": ");
-
-        buffer_sprintf(
-            wb,
-            "\t\t{\n"
-            "\t\t\t\"id\": \"%s\",\n"
-            "\t\t\t\"name\": \"%s\",\n"
-            "\t\t\t\"type\": \"%s\",\n"
-            "\t\t\t\"family\": \"%s\",\n"
-            "\t\t\t\"context\": \"%s\",\n"
-            "\t\t\t\"title\": \"%s (%s)\",\n"
-            "\t\t\t\"priority\": %ld,\n"
-            "\t\t\t\"plugin\": \"%s\",\n"
-            "\t\t\t\"module\": \"%s\",\n"
-            "\t\t\t\"enabled\": %s,\n"
-            "\t\t\t\"units\": \"%s\",\n"
-            "\t\t\t\"data_url\": \"/api/v1/data?chart=%s\",\n"
-            "\t\t\t\"chart_type\": \"%s\",\n",
-            id //sqlite3_column_text(res_chart, 1)
-            ,
-            id // sqlite3_column_text(res_chart, 2)
-            ,
-            sqlite3_column_text(res_chart, 3), sqlite3_column_text(res_chart, 4), sqlite3_column_text(res_chart, 5),
-            sqlite3_column_text(res_chart, 6), id //sqlite3_column_text(res_chart, 2)
-            ,
-            (long ) sqlite3_column_int(res_chart, 7),
-            (const char *) sqlite3_column_text(res_chart, 8) ? (const char *) sqlite3_column_text(res_chart, 8) : (char *) "",
-            (const char *) sqlite3_column_text(res_chart, 9) ? (const char *) sqlite3_column_text(res_chart, 9) : (char *) "", (char *) "false",
-            (const char *) sqlite3_column_text(res_chart, 10), id //sqlite3_column_text(res_chart, 2)
-            ,
-            rrdset_type_name(sqlite3_column_int(res_chart, 11)));
-
-        sql_rrdim2json(res_dim, (uuid_t *) sqlite3_column_blob(res_chart, 0), wb, &dimensions);
-
-        rc = sqlite3_reset(res_dim);
-        if (unlikely(rc != SQLITE_OK))
-            error_report("Failed to reset the prepared statement when reading archived chart dimensions");
-        buffer_strcat(wb, "\n\t\t}");
-    }
-
-    buffer_sprintf(wb
-        , "\n\t}"
-          ",\n\t\"charts_count\": %zu"
-          ",\n\t\"dimensions_count\": %zu"
-          ",\n\t\"alarms_count\": %zu"
-          ",\n\t\"rrd_memory_bytes\": %zu"
-          ",\n\t\"hosts_count\": %zu"
-          ",\n\t\"hosts\": ["
-        , c
-        , dimensions
-        , (size_t) 0
-        , (size_t) 0
-        , rrd_hosts_available
-    );
-
-    if(unlikely(rrd_hosts_available > 1)) {
-        rrd_rdlock();
-
-        size_t found = 0;
-        RRDHOST *h;
-        rrdhost_foreach_read(h) {
-            if(!rrdhost_should_be_removed(h, host, now) && !rrdhost_flag_check(h, RRDHOST_FLAG_ARCHIVED)) {
-                buffer_sprintf(wb
-                    , "%s\n\t\t{"
-                      "\n\t\t\t\"hostname\": \"%s\""
-                      "\n\t\t}"
-                    , (found > 0) ? "," : ""
-                    , rrdhost_hostname(h)
-                );
-
-                found++;
-            }
-        }
-
-        rrd_unlock();
-    }
-    else {
-        buffer_sprintf(wb
-            , "\n\t\t{"
-              "\n\t\t\t\"hostname\": \"%s\""
-              "\n\t\t}"
-            , rrdhost_hostname(host)
-        );
-    }
-
-    buffer_sprintf(wb, "\n\t]\n}\n");
-
-    rc = sqlite3_finalize(res_dim);
-    if (unlikely(rc != SQLITE_OK))
-        error_report("Failed to finalize the prepared statement when reading archived chart dimensions");
-
-failed:
-    rc = sqlite3_finalize(res_chart);
-    if (unlikely(rc != SQLITE_OK))
-        error_report("Failed to finalize the prepared statement when reading archived charts");
-}
-
-void db_execute(const char *cmd)
+// Return 0 OK
+// Return 1 Failed
+int db_execute(sqlite3 *db, const char *cmd)
 {
     int rc;
     int cnt = 0;
     while (cnt < SQL_MAX_RETRY) {
         char *err_msg;
-        rc = sqlite3_exec_monitored(db_meta, cmd, 0, 0, &err_msg);
+        rc = sqlite3_exec_monitored(db, cmd, 0, 0, &err_msg);
         if (rc != SQLITE_OK) {
             error_report("Failed to execute '%s', rc = %d (%s) -- attempt %d", cmd, rc, err_msg, cnt);
             sqlite3_free(err_msg);
@@ -730,116 +582,7 @@ void db_execute(const char *cmd)
 
         ++cnt;
     }
-}
-
-#define SELECT_MIGRATED_FILE    "select 1 from metadata_migration where filename = @path;"
-
-int file_is_migrated(char *path)
-{
-    sqlite3_stmt *res = NULL;
-    int rc;
-
-    rc = sqlite3_prepare_v2(db_meta, SELECT_MIGRATED_FILE, -1, &res, 0);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to prepare statement to fetch host");
-        return 0;
-    }
-
-    rc = sqlite3_bind_text(res, 1, path, -1, SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to bind filename parameter to check migration");
-        return 0;
-    }
-
-    rc = sqlite3_step_monitored(res);
-
-    if (unlikely(sqlite3_finalize(res) != SQLITE_OK))
-        error_report("Failed to finalize the prepared statement when checking if metadata file is migrated");
-
-    return (rc == SQLITE_ROW);
-}
-
-#define STORE_MIGRATED_FILE    "insert or replace into metadata_migration (filename, file_size, date_created) " \
-                                "values (@file, @size, unixepoch());"
-
-void add_migrated_file(char *path, uint64_t file_size)
-{
-    sqlite3_stmt *res = NULL;
-    int rc;
-
-    rc = sqlite3_prepare_v2(db_meta, STORE_MIGRATED_FILE, -1, &res, 0);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to prepare statement to fetch host");
-        return;
-    }
-
-    rc = sqlite3_bind_text(res, 1, path, -1, SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to bind filename parameter to store migration information");
-        return;
-    }
-
-    rc = sqlite3_bind_int64(res, 2, (sqlite_int64) file_size);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to bind size parameter to store migration information");
-        return;
-    }
-
-    rc = execute_insert(res);
-    if (unlikely(rc != SQLITE_DONE))
-        error_report("Failed to store migrated file, rc = %d", rc);
-
-    if (unlikely(sqlite3_finalize(res) != SQLITE_OK))
-        error_report("Failed to finalize the prepared statement when checking if metadata file is migrated");
-}
-
-
-
-#define SQL_STORE_CLAIM_ID  "insert into node_instance " \
-    "(host_id, claim_id, date_created) values (@host_id, @claim_id, unixepoch()) " \
-    "on conflict(host_id) do update set claim_id = excluded.claim_id;"
-
-void store_claim_id(uuid_t *host_id, uuid_t *claim_id)
-{
-    sqlite3_stmt *res = NULL;
-    int rc;
-
-    if (unlikely(!db_meta)) {
-        if (default_rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE)
-            error_report("Database has not been initialized");
-        return;
-    }
-
-    rc = sqlite3_prepare_v2(db_meta, SQL_STORE_CLAIM_ID, -1, &res, 0);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to prepare statement store chart labels");
-        return;
-    }
-
-    rc = sqlite3_bind_blob(res, 1, host_id, sizeof(*host_id), SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to bind host_id parameter to store node instance information");
-        goto failed;
-    }
-
-    if (claim_id)
-        rc = sqlite3_bind_blob(res, 2, claim_id, sizeof(*claim_id), SQLITE_STATIC);
-    else
-        rc = sqlite3_bind_null(res, 2);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to bind claim_id parameter to store node instance information");
-        goto failed;
-    }
-
-    rc = execute_insert(res);
-    if (unlikely(rc != SQLITE_DONE))
-        error_report("Failed to store node instance information, rc = %d", rc);
-
-failed:
-    if (unlikely(sqlite3_finalize(res) != SQLITE_OK))
-        error_report("Failed to finalize the prepared statement when storing node instance information");
-
-    return;
+    return (rc != SQLITE_OK);
 }
 
 static inline void set_host_node_id(RRDHOST *host, uuid_t *node_id)
@@ -849,15 +592,20 @@ static inline void set_host_node_id(RRDHOST *host, uuid_t *node_id)
 
     if (unlikely(!node_id)) {
         freez(host->node_id);
-        host->node_id = NULL;
+        __atomic_store_n(&host->node_id, NULL, __ATOMIC_RELAXED);
         return;
     }
 
-    struct aclk_database_worker_config *wc = host->dbsync_worker;
+    struct aclk_sync_host_config *wc = host->aclk_sync_host_config;
 
-    if (unlikely(!host->node_id))
-        host->node_id = mallocz(sizeof(*host->node_id));
-    uuid_copy(*(host->node_id), *node_id);
+    if (unlikely(!host->node_id)) {
+        uuid_t *t = mallocz(sizeof(*host->node_id));
+        uuid_copy(*t, *node_id);
+        __atomic_store_n(&host->node_id, t, __ATOMIC_RELAXED);
+    }
+    else {
+        uuid_copy(*(host->node_id), *node_id);
+    }
 
     if (unlikely(!wc))
         sql_create_aclk_table(host, &host->host_uuid, node_id);
@@ -873,6 +621,14 @@ int update_node_id(uuid_t *host_id, uuid_t *node_id)
     RRDHOST *host = NULL;
     int rc = 2;
 
+    char host_guid[GUID_LEN + 1];
+    uuid_unparse_lower(*host_id, host_guid);
+    rrd_wrlock();
+    host = rrdhost_find_by_guid(host_guid);
+    if (likely(host))
+        set_host_node_id(host, node_id);
+    rrd_unlock();
+
     if (unlikely(!db_meta)) {
         if (default_rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE)
             error_report("Database has not been initialized");
@@ -902,14 +658,6 @@ int update_node_id(uuid_t *host_id, uuid_t *node_id)
         error_report("Failed to store node instance information, rc = %d", rc);
     rc = sqlite3_changes(db_meta);
 
-    char host_guid[GUID_LEN + 1];
-    uuid_unparse_lower(*host_id, host_guid);
-    rrd_wrlock();
-    host = rrdhost_find_by_guid(host_guid);
-    if (likely(host))
-            set_host_node_id(host, node_id);
-    rrd_unlock();
-
 failed:
     if (unlikely(sqlite3_finalize(res) != SQLITE_OK))
         error_report("Failed to finalize the prepared statement when storing node instance information");
@@ -917,48 +665,6 @@ failed:
     return rc - 1;
 }
 
-#define SQL_SELECT_HOSTNAME_BY_NODE_ID  "SELECT h.hostname FROM node_instance ni, " \
-"host h WHERE ni.host_id = h.host_id AND ni.node_id = @node_id;"
-
-char *get_hostname_by_node_id(char *node)
-{
-    sqlite3_stmt *res = NULL;
-    char  *hostname = NULL;
-    int rc;
-
-    if (unlikely(!db_meta)) {
-        if (default_rrd_memory_mode == RRD_MEMORY_MODE_DBENGINE)
-            error_report("Database has not been initialized");
-        return NULL;
-    }
-
-    uuid_t node_id;
-    if (uuid_parse(node, node_id))
-        return NULL;
-
-    rc = sqlite3_prepare_v2(db_meta, SQL_SELECT_HOSTNAME_BY_NODE_ID, -1, &res, 0);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to prepare statement to fetch hostname by node id");
-        return NULL;
-    }
-
-    rc = sqlite3_bind_blob(res, 1, &node_id, sizeof(node_id), SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK)) {
-        error_report("Failed to bind host_id parameter to select node instance information");
-        goto failed;
-    }
-
-    rc = sqlite3_step_monitored(res);
-    if (likely(rc == SQLITE_ROW))
-        hostname = strdupz((char *)sqlite3_column_text(res, 0));
-
-failed:
-    if (unlikely(sqlite3_finalize(res) != SQLITE_OK))
-        error_report("Failed to finalize the prepared statement when search for hostname by node id");
-
-    return hostname;
-}
-
 #define SQL_SELECT_HOST_BY_NODE_ID  "select host_id from node_instance where node_id = @node_id;"
 
 int get_host_id(uuid_t *node_id, uuid_t *host_id)
@@ -997,7 +703,7 @@ failed:
     return (rc == SQLITE_ROW) ? 0 : -1;
 }
 
-#define SQL_SELECT_NODE_ID  "select node_id from node_instance where host_id = @host_id and node_id not null;"
+#define SQL_SELECT_NODE_ID  "SELECT node_id FROM node_instance WHERE host_id = @host_id AND node_id IS NOT NULL;"
 
 int get_node_id(uuid_t *host_id, uuid_t *node_id)
 {
@@ -1033,8 +739,8 @@ failed:
     return (rc == SQLITE_ROW) ? 0 : -1;
 }
 
-#define SQL_INVALIDATE_NODE_INSTANCES "update node_instance set node_id = NULL where exists " \
-    "(select host_id from node_instance where host_id = @host_id and (@claim_id is null or claim_id <> @claim_id));"
+#define SQL_INVALIDATE_NODE_INSTANCES "UPDATE node_instance SET node_id = NULL WHERE EXISTS " \
+    "(SELECT host_id FROM node_instance WHERE host_id = @host_id AND (@claim_id IS NULL OR claim_id <> @claim_id));"
 
 void invalidate_node_instances(uuid_t *host_id, uuid_t *claim_id)
 {
@@ -1078,8 +784,8 @@ failed:
         error_report("Failed to finalize the prepared statement when invalidating node instance information");
 }
 
-#define SQL_GET_NODE_INSTANCE_LIST "select ni.node_id, ni.host_id, h.hostname " \
-    "from node_instance ni, host h where ni.host_id = h.host_id;"
+#define SQL_GET_NODE_INSTANCE_LIST "SELECT ni.node_id, ni.host_id, h.hostname " \
+    "FROM node_instance ni, host h WHERE ni.host_id = h.host_id AND h.hops >=0;"
 
 struct node_instance_list *get_node_list(void)
 {
@@ -1118,13 +824,18 @@ struct node_instance_list *get_node_list(void)
             uuid_copy(node_list[row].node_id, *((uuid_t *)sqlite3_column_blob(res, 0)));
         if (sqlite3_column_bytes(res, 1) == sizeof(uuid_t)) {
             uuid_t *host_id = (uuid_t *)sqlite3_column_blob(res, 1);
-            uuid_copy(node_list[row].host_id, *host_id);
-            node_list[row].queryable = 1;
             uuid_unparse_lower(*host_id, host_guid);
             RRDHOST *host = rrdhost_find_by_guid(host_guid);
-            node_list[row].live = host && (host == localhost || host->receiver) ? 1 : 0;
+            if (rrdhost_flag_check(host, RRDHOST_FLAG_PENDING_CONTEXT_LOAD)) {
+                netdata_log_info("ACLK: 'host:%s' skipping get node list because context is initializing", rrdhost_hostname(host));
+                continue;
+            }
+            uuid_copy(node_list[row].host_id, *host_id);
+            node_list[row].queryable = 1;
+            node_list[row].live = (host && (host == localhost || host->receiver
+                                            || !(rrdhost_flag_check(host, RRDHOST_FLAG_ORPHAN)))) ? 1 : 0;
             node_list[row].hops = (host && host->system_info) ? host->system_info->hops :
-                                  uuid_compare(*host_id, localhost->host_uuid) ? 1 : 0;
+                                  uuid_memcmp(host_id, &localhost->host_uuid) ? 1 : 0;
             node_list[row].hostname =
                 sqlite3_column_bytes(res, 2) ? strdupz((char *)sqlite3_column_text(res, 2)) : NULL;
         }
@@ -1263,117 +974,28 @@ int bind_text_null(sqlite3_stmt *res, int position, const char *text, bool can_b
 int sql_metadata_cache_stats(int op)
 {
     int count, dummy;
+
+    if (unlikely(!db_meta))
+        return 0;
+
     netdata_thread_disable_cancelability();
     sqlite3_db_status(db_meta, op, &count, &dummy, 0);
     netdata_thread_enable_cancelability();
     return count;
 }
 
-#define SQL_FIND_CHART_UUID                                                                                            \
-    "SELECT chart_id FROM chart WHERE host_id = @host AND type=@type AND id=@id AND (name IS NULL OR name=@name) AND chart_id IS NOT NULL;"
+#define SQL_DROP_TABLE "DROP table %s;"
 
-#define SQL_FIND_DIMENSION_UUID \
-    "SELECT dim_id FROM dimension WHERE chart_id=@chart AND id=@id AND name=@name AND LENGTH(dim_id)=16;"
-
-
-//Do a database lookup to find the UUID of a chart
-//If found store it in store_uuid and return 0
-int sql_find_chart_uuid(RRDHOST *host, RRDSET *st, uuid_t *store_uuid)
+void sql_drop_table(const char *table)
 {
-    static __thread sqlite3_stmt *res = NULL;
-    int rc;
+    if (!table)
+        return;
 
-    const char *name = string2str(st->parts.name);
+    char wstr[255];
+    snprintfz(wstr, 254, SQL_DROP_TABLE, table);
 
-    if (unlikely(!db_meta) && default_rrd_memory_mode != RRD_MEMORY_MODE_DBENGINE)
-        return 1;
-
-    if (unlikely(!res)) {
-        rc = prepare_statement(db_meta, SQL_FIND_CHART_UUID, &res);
-        if (rc != SQLITE_OK) {
-            error_report("Failed to prepare statement to lookup chart UUID in the database");
-            return 1;
-        }
+    int rc = sqlite3_exec_monitored(db_meta, wstr, 0, 0, NULL);
+    if (rc != SQLITE_OK) {
+        error_report("DES SQLite error during drop table operation for %s, rc = %d", table, rc);
     }
-
-    rc = sqlite3_bind_blob(res, 1, &host->host_uuid, sizeof(host->host_uuid), SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK))
-        goto bind_fail;
-
-    rc = sqlite3_bind_text(res, 2,  string2str(st->parts.type), -1, SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK))
-        goto bind_fail;
-
-    rc = sqlite3_bind_text(res, 3, string2str(st->parts.id), -1, SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK))
-        goto bind_fail;
-
-    rc = sqlite3_bind_text(res, 4, name && *name ? name : string2str(st->parts.id), -1, SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK))
-        goto bind_fail;
-
-    int status = 1;
-    rc = sqlite3_step_monitored(res);
-    if (likely(rc == SQLITE_ROW)) {
-        uuid_copy(*store_uuid, sqlite3_column_blob(res, 0));
-        status = 0;
-    }
-
-    rc = sqlite3_reset(res);
-    if (unlikely(rc != SQLITE_OK))
-        error_report("Failed to reset statement when searching for a chart UUID, rc = %d", rc);
-
-    return status;
-
-bind_fail:
-    error_report("Failed to bind input parameter to perform chart UUID database lookup, rc = %d", rc);
-    rc = sqlite3_reset(res);
-    if (unlikely(rc != SQLITE_OK))
-        error_report("Failed to reset statement when searching for a chart UUID, rc = %d", rc);
-    return 1;
-}
-
-int sql_find_dimension_uuid(RRDSET *st, RRDDIM *rd, uuid_t *store_uuid)
-{
-    static __thread sqlite3_stmt *res = NULL;
-    int rc;
-    int status = 1;
-
-    if (unlikely(!db_meta) && default_rrd_memory_mode != RRD_MEMORY_MODE_DBENGINE)
-        return 1;
-
-    if (unlikely(!res)) {
-        rc = prepare_statement(db_meta, SQL_FIND_DIMENSION_UUID, &res);
-        if (rc != SQLITE_OK) {
-            error_report("Failed to bind prepare statement to lookup dimension UUID in the database");
-            return 1;
-        }
-    }
-
-    rc = sqlite3_bind_blob(res, 1, st->chart_uuid, sizeof(*st->chart_uuid), SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK))
-        goto bind_fail;
-
-    rc = sqlite3_bind_text(res, 2, rrddim_id(rd), -1, SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK))
-        goto bind_fail;
-
-    rc = sqlite3_bind_text(res, 3, rrddim_name(rd), -1, SQLITE_STATIC);
-    if (unlikely(rc != SQLITE_OK))
-        goto bind_fail;
-
-    rc = sqlite3_step_monitored(res);
-    if (likely(rc == SQLITE_ROW)) {
-        uuid_copy(*store_uuid, *((uuid_t *) sqlite3_column_blob(res, 0)));
-        status = 0;
-    }
-
-    rc = sqlite3_reset(res);
-    if (unlikely(rc != SQLITE_OK))
-        error_report("Failed to reset statement find dimension uuid, rc = %d", rc);
-    return status;
-
-bind_fail:
-    error_report("Failed to bind input parameter to perform dimension UUID database lookup, rc = %d", rc);
-    return 1;
 }
