@@ -54,10 +54,9 @@ void sql_close_database(void);
 int bind_text_null(sqlite3_stmt *res, int position, const char *text, bool can_be_null);
 int prepare_statement(sqlite3 *database, const char *query, sqlite3_stmt **statement);
 int execute_insert(sqlite3_stmt *res);
-int file_is_migrated(char *path);
 int exec_statement_with_uuid(const char *sql, uuid_t *uuid);
-void add_migrated_file(char *path, uint64_t file_size);
-void db_execute(const char *cmd);
+int db_execute(sqlite3 *database, const char *cmd);
+void initialize_thread_key_pool(void);
 
 // Look up functions
 int get_node_id(uuid_t *host_id, uuid_t *node_id);
@@ -65,16 +64,11 @@ int get_host_id(uuid_t *node_id, uuid_t *host_id);
 struct node_instance_list *get_node_list(void);
 void sql_load_node_id(RRDHOST *host);
 char *get_hostname_by_node_id(char *node_id);
-int sql_find_chart_uuid(RRDHOST *host, RRDSET *st, uuid_t *store_uuid);
-int sql_find_dimension_uuid(RRDSET *st, RRDDIM *rd, uuid_t *store_uuid);
 
 // Help build archived hosts in memory when agent starts
 void sql_build_host_system_info(uuid_t *host_id, struct rrdhost_system_info *system_info);
 DICTIONARY *sql_load_host_labels(uuid_t *host_id);
 
-// For queries: To be removed when context queries are implemented
-void sql_rrdset2json(RRDHOST *host, BUFFER *wb);
-
 // TODO: move to metadata
 int update_node_id(uuid_t *host_id, uuid_t *node_id);
 
@@ -83,4 +77,6 @@ void invalidate_node_instances(uuid_t *host_id, uuid_t *claim_id);
 // Provide statistics
 int sql_metadata_cache_stats(int op);
 
+void sql_drop_table(const char *table);
+void sqlite_now_usec(sqlite3_context *context, int argc, sqlite3_value **argv);
 #endif //NETDATA_SQLITE_FUNCTIONS_H
