@@ -7,18 +7,21 @@
 
 extern unsigned int default_health_enabled;
 
-#define HEALTH_ENTRY_FLAG_PROCESSED             0x00000001
-#define HEALTH_ENTRY_FLAG_UPDATED               0x00000002
-#define HEALTH_ENTRY_FLAG_EXEC_RUN              0x00000004
-#define HEALTH_ENTRY_FLAG_EXEC_FAILED           0x00000008
-#define HEALTH_ENTRY_FLAG_SILENCED              0x00000010
-#define HEALTH_ENTRY_RUN_ONCE                   0x00000020
-#define HEALTH_ENTRY_FLAG_EXEC_IN_PROGRESS      0x00000040
-#define HEALTH_ENTRY_FLAG_IS_REPEATING          0x00000080
+typedef enum __attribute__((packed)) {
+    HEALTH_ENTRY_FLAG_PROCESSED             = 0x00000001, // notifications engine has processed this
+    HEALTH_ENTRY_FLAG_UPDATED               = 0x00000002, // there is a more recent update about this transition
+    HEALTH_ENTRY_FLAG_EXEC_RUN              = 0x00000004, // notification script has been run (this is the intent, not the result)
+    HEALTH_ENTRY_FLAG_EXEC_FAILED           = 0x00000008, // notification script couldn't be run
+    HEALTH_ENTRY_FLAG_SILENCED              = 0x00000010,
+    HEALTH_ENTRY_RUN_ONCE                   = 0x00000020,
+    HEALTH_ENTRY_FLAG_EXEC_IN_PROGRESS      = 0x00000040,
+    HEALTH_ENTRY_FLAG_IS_REPEATING          = 0x00000080,
+    HEALTH_ENTRY_FLAG_SAVED                 = 0x10000000, // Saved to SQL
+    HEALTH_ENTRY_FLAG_ACLK_QUEUED           = 0x20000000, // Sent to Netdata Cloud
+    HEALTH_ENTRY_FLAG_NO_CLEAR_NOTIFICATION = 0x80000000,
+} HEALTH_ENTRY_FLAGS;
 
-#define HEALTH_ENTRY_FLAG_SAVED                 0x10000000
-#define HEALTH_ENTRY_FLAG_ACLK_QUEUED           0x20000000
-#define HEALTH_ENTRY_FLAG_NO_CLEAR_NOTIFICATION 0x80000000
+void health_entry_flags_to_json_array(BUFFER *wb, const char *key, HEALTH_ENTRY_FLAGS flags);
 
 #ifndef HEALTH_LISTEN_PORT
 #define HEALTH_LISTEN_PORT 19998
@@ -28,9 +31,19 @@ extern unsigned int default_health_enabled;
 #define HEALTH_LISTEN_BACKLOG 4096
 #endif
 
+#ifndef HEALTH_LOG_DEFAULT_HISTORY
+#define HEALTH_LOG_DEFAULT_HISTORY 432000
+#endif
+
+#ifndef HEALTH_LOG_MINIMUM_HISTORY
+#define HEALTH_LOG_MINIMUM_HISTORY 86400
+#endif
+
 #define HEALTH_SILENCERS_MAX_FILE_LEN 10000
 
 extern char *silencers_filename;
+extern SIMPLE_PATTERN *conf_enabled_alarms;
+extern DICTIONARY *health_rrdvars;
 
 void health_init(void);
 
@@ -38,8 +51,8 @@ void health_reload(void);
 
 void health_aggregate_alarms(RRDHOST *host, BUFFER *wb, BUFFER* context, RRDCALC_STATUS status);
 void health_alarms2json(RRDHOST *host, BUFFER *wb, int all);
+void health_alert2json_conf(RRDHOST *host, BUFFER *wb, CONTEXTS_V2_OPTIONS all);
 void health_alarms_values2json(RRDHOST *host, BUFFER *wb, int all);
-void health_alarm_log2json(RRDHOST *host, BUFFER *wb, uint32_t after, char *chart);
 
 void health_api_v1_chart_variables2json(RRDSET *st, BUFFER *buf);
 void health_api_v1_chart_custom_variables2json(RRDSET *st, BUFFER *buf);
@@ -48,9 +61,6 @@ int health_alarm_log_open(RRDHOST *host);
 void health_alarm_log_save(RRDHOST *host, ALARM_ENTRY *ae);
 void health_alarm_log_load(RRDHOST *host);
 
-void health_thread_spawn(RRDHOST *host);
-void health_thread_stop(RRDHOST *host);
-
 ALARM_ENTRY* health_create_alarm_entry(
     RRDHOST *host,
     uint32_t alarm_id,
@@ -60,6 +70,7 @@ ALARM_ENTRY* health_create_alarm_entry(
     STRING *name,
     STRING *chart,
     STRING *chart_context,
+    STRING *chart_id,
     STRING *family,
     STRING *classification,
     STRING *component,
@@ -75,15 +86,10 @@ ALARM_ENTRY* health_create_alarm_entry(
     STRING *units,
     STRING *info,
     int delay,
-    uint32_t flags);
+    HEALTH_ENTRY_FLAGS flags);
 
 void health_alarm_log_add_entry(RRDHOST *host, ALARM_ENTRY *ae);
 
-struct health_state {
-    RRDHOST *host;
-    netdata_thread_t thread;
-};
-
 void health_readdir(RRDHOST *host, const char *user_path, const char *stock_path, const char *subpath);
 char *health_user_config_dir(void);
 char *health_stock_config_dir(void);
@@ -93,11 +99,10 @@ void health_alarm_log_free_one_nochecks_nounlink(ALARM_ENTRY *ae);
 
 void *health_cmdapi_thread(void *ptr);
 
-void health_label_log_save(RRDHOST *host);
-
 char *health_edit_command_from_source(const char *source);
 void sql_refresh_hashes(void);
 
 void health_add_host_labels(void);
+void health_string2json(BUFFER *wb, const char *prefix, const char *label, const char *value, const char *suffix);
 
 #endif //NETDATA_HEALTH_H
