@@ -4,9 +4,14 @@
 #include "registry_internals.h"
 
 #define REGISTRY_STATUS_OK "ok"
+#define REGISTRY_STATUS_REDIRECT "redirect"
 #define REGISTRY_STATUS_FAILED "failed"
 #define REGISTRY_STATUS_DISABLED "disabled"
 
+bool registry_is_valid_url(const char *url) {
+    return url && (*url == 'h' || *url == '*');
+}
+
 // ----------------------------------------------------------------------------
 // REGISTRY concurrency locking
 
@@ -18,35 +23,28 @@ static inline void registry_unlock(void) {
     netdata_mutex_unlock(&registry.lock);
 }
 
-
 // ----------------------------------------------------------------------------
 // COOKIES
 
 static void registry_set_cookie(struct web_client *w, const char *guid) {
-    char edate[100], domain[512];
+    char e_date[100];
     time_t et = now_realtime_sec() + registry.persons_expiration;
-    struct tm etmbuf, *etm = gmtime_r(&et, &etmbuf);
-    strftime(edate, sizeof(edate), "%a, %d %b %Y %H:%M:%S %Z", etm);
+    struct tm e_tm_buf, *etm = gmtime_r(&et, &e_tm_buf);
+    strftime(e_date, sizeof(e_date), "%a, %d %b %Y %H:%M:%S %Z", etm);
 
-    snprintfz(w->cookie1, NETDATA_WEB_REQUEST_COOKIE_SIZE, NETDATA_REGISTRY_COOKIE_NAME "=%s; Expires=%s", guid, edate);
+    buffer_sprintf(w->response.header, "Set-Cookie: " NETDATA_REGISTRY_COOKIE_NAME "=%s; Expires=%s\r\n", guid, e_date);
+    buffer_sprintf(w->response.header, "Set-Cookie: " NETDATA_REGISTRY_COOKIE_NAME "=%s; SameSite=Strict; Expires=%s\r\n", guid, e_date);
+    if(registry.enable_cookies_samesite_secure)
+        buffer_sprintf(w->response.header, "Set-Cookie: " NETDATA_REGISTRY_COOKIE_NAME "=%s; Expires=%s; SameSite=None; Secure\r\n", guid, e_date);
 
-    if(registry.registry_domain && registry.registry_domain[0])
-        snprintfz(domain, 511, "Domain=%s", registry.registry_domain);
-    else
-        domain[0]='\0';
-
-    int length = snprintfz(w->cookie2, NETDATA_WEB_REQUEST_COOKIE_SIZE,
-                           NETDATA_REGISTRY_COOKIE_NAME "=%s; Expires=%s; %s",
-                           guid, edate, domain);
-
-    size_t remaining_length = NETDATA_WEB_REQUEST_COOKIE_SIZE - length;
-    // 25 is the necessary length to add new cookies
-    if (registry.enable_cookies_samesite_secure) {
-        if (length > 0 && remaining_length > 25)
-            snprintfz(&w->cookie2[length], remaining_length, "; SameSite=None; Secure");
-        else
-            error("Netdata does not have enough space to store cookies SameSite and Secure");
+    if(registry.registry_domain && *registry.registry_domain) {
+        buffer_sprintf(w->response.header, "Set-Cookie: " NETDATA_REGISTRY_COOKIE_NAME "=%s; Expires=%s; Domain=%s\r\n", guid, e_date, registry.registry_domain);
+        buffer_sprintf(w->response.header, "Set-Cookie: " NETDATA_REGISTRY_COOKIE_NAME "=%s; Expires=%s; Domain=%s; SameSite=Strict\r\n", guid, e_date, registry.registry_domain);
+        if(registry.enable_cookies_samesite_secure)
+            buffer_sprintf(w->response.header, "Set-Cookie: " NETDATA_REGISTRY_COOKIE_NAME "=%s; Expires=%s; Domain=%s; SameSite=None; Secure\r\n", guid, e_date, registry.registry_domain);
     }
+
+    w->response.has_cookies = true;
 }
 
 static inline void registry_set_person_cookie(struct web_client *w, REGISTRY_PERSON *p) {
@@ -59,23 +57,25 @@ static inline void registry_set_person_cookie(struct web_client *w, REGISTRY_PER
 
 static inline void registry_json_header(RRDHOST *host, struct web_client *w, const char *action, const char *status) {
     buffer_flush(w->response.data);
-    w->response.data->contenttype = CT_APPLICATION_JSON;
-    buffer_sprintf(w->response.data, "{\n\t\"action\": \"%s\",\n\t\"status\": \"%s\",\n\t\"hostname\": \"%s\",\n\t\"machine_guid\": \"%s\"",
-            action, status, rrdhost_registry_hostname(host), host->machine_guid);
+    w->response.data->content_type = CT_APPLICATION_JSON;
+    buffer_json_initialize(w->response.data, "\"", "\"", 0, true, BUFFER_JSON_OPTIONS_DEFAULT);
+    buffer_json_member_add_string(w->response.data, "action", action);
+    buffer_json_member_add_string(w->response.data, "status", status);
+    buffer_json_member_add_string(w->response.data, "hostname", rrdhost_registry_hostname(host));
+    buffer_json_member_add_string(w->response.data, "machine_guid", host->machine_guid);
 }
 
 static inline void registry_json_footer(struct web_client *w) {
-    buffer_strcat(w->response.data, "\n}\n");
+    buffer_json_finalize(w->response.data);
 }
 
 static inline int registry_json_disabled(RRDHOST *host, struct web_client *w, const char *action) {
     registry_json_header(host, w, action, REGISTRY_STATUS_DISABLED);
 
-    buffer_sprintf(w->response.data, ",\n\t\"registry\": \"%s\"",
-            registry.registry_to_announce);
+    buffer_json_member_add_string(w->response.data, "registry", registry.registry_to_announce);
 
     registry_json_footer(w);
-    return 200;
+    return HTTP_RESP_OK;
 }
 
 
@@ -90,37 +90,45 @@ struct registry_json_walk_person_urls_callback {
     int count;
 };
 
+static STRING *asterisks = NULL;
+
 // callback for rendering PERSON_URLs
-static int registry_json_person_url_callback(void *entry, void *data) {
-    REGISTRY_PERSON_URL *pu = (REGISTRY_PERSON_URL *)entry;
-    struct registry_json_walk_person_urls_callback *c = (struct registry_json_walk_person_urls_callback *)data;
+static int registry_json_person_url_callback(REGISTRY_PERSON_URL *pu, struct registry_json_walk_person_urls_callback *c) {
+    if(unlikely(!asterisks))
+        asterisks = string_strdupz("***");
+
     struct web_client *w = c->w;
 
-    if (!strcmp(pu->url->url,"***")) return 0;
-    
-    if(unlikely(c->count++))
-        buffer_strcat(w->response.data, ",");
+    if (pu->url == asterisks) return 0;
 
-    buffer_sprintf(w->response.data, "\n\t\t[ \"%s\", \"%s\", %u000, %u, \"%s\" ]",
-            pu->machine->guid, pu->url->url, pu->last_t, pu->usages, pu->machine_name);
+    buffer_json_add_array_item_array(w->response.data);
+    buffer_json_add_array_item_string(w->response.data, pu->machine->guid);
+    buffer_json_add_array_item_string(w->response.data, string2str(pu->url));
+    buffer_json_add_array_item_uint64(w->response.data, pu->last_t * (uint64_t) 1000);
+    buffer_json_add_array_item_uint64(w->response.data, pu->usages);
+    buffer_json_add_array_item_string(w->response.data, string2str(pu->machine_name));
+    buffer_json_array_close(w->response.data);
 
-    return 0;
+    return 1;
 }
 
 // callback for rendering MACHINE_URLs
-static int registry_json_machine_url_callback(const DICTIONARY_ITEM *item __maybe_unused, void *entry, void *data) {
-    REGISTRY_MACHINE_URL *mu = (REGISTRY_MACHINE_URL *)entry;
-    struct registry_json_walk_person_urls_callback *c = (struct registry_json_walk_person_urls_callback *)data;
+static int registry_json_machine_url_callback(REGISTRY_MACHINE_URL *mu, struct registry_json_walk_person_urls_callback *c, STRING *hostname) {
+    if(unlikely(!asterisks))
+        asterisks = string_strdupz("***");
+
     struct web_client *w = c->w;
     REGISTRY_MACHINE *m = c->m;
 
-    if (!strcmp(mu->url->url,"***")) return 1;
+    if (mu->url == asterisks) return 0;
 
-    if(unlikely(c->count++))
-        buffer_strcat(w->response.data, ",");
-
-    buffer_sprintf(w->response.data, "\n\t\t[ \"%s\", \"%s\", %u000, %u ]",
-            m->guid, mu->url->url, mu->last_t, mu->usages);
+    buffer_json_add_array_item_array(w->response.data);
+    buffer_json_add_array_item_string(w->response.data, m->guid);
+    buffer_json_add_array_item_string(w->response.data, string2str(mu->url));
+    buffer_json_add_array_item_uint64(w->response.data, mu->last_t * (uint64_t) 1000);
+    buffer_json_add_array_item_uint64(w->response.data, mu->usages);
+    buffer_json_add_array_item_string(w->response.data, string2str(hostname));
+    buffer_json_array_close(w->response.data);
 
     return 1;
 }
@@ -133,9 +141,7 @@ struct registry_person_url_callback_verify_machine_exists_data {
     int count;
 };
 
-static inline int registry_person_url_callback_verify_machine_exists(void *entry, void *data) {
-    struct registry_person_url_callback_verify_machine_exists_data *d = (struct registry_person_url_callback_verify_machine_exists_data *)data;
-    REGISTRY_PERSON_URL *pu = (REGISTRY_PERSON_URL *)entry;
+static inline int registry_person_url_callback_verify_machine_exists(REGISTRY_PERSON_URL *pu, struct registry_person_url_callback_verify_machine_exists_data *d) {
     REGISTRY_MACHINE *m = d->m;
 
     if(pu->machine == m)
@@ -149,54 +155,99 @@ static inline int registry_person_url_callback_verify_machine_exists(void *entry
 // The registry does not seem to be designed to support this and I cannot see any concurrency protection
 // that could make this safe, so try to be as atomic as possible.
 
-void registry_update_cloud_base_url()
-{
-    // This is guaranteed to be set early in main via post_conf_load()
-    registry.cloud_base_url = appconfig_get(&cloud_config, CONFIG_SECTION_GLOBAL, "cloud base url", NULL);
-    if (registry.cloud_base_url == NULL)
-        fatal("Do not move the cloud base url out of post_conf_load!!");
-
+void registry_update_cloud_base_url() {
+    registry.cloud_base_url = appconfig_get(&cloud_config, CONFIG_SECTION_GLOBAL, "cloud base url", DEFAULT_CLOUD_BASE_URL);
     setenv("NETDATA_REGISTRY_CLOUD_BASE_URL", registry.cloud_base_url, 1);
 }
+
 // ----------------------------------------------------------------------------
 // public HELLO request
 
 int registry_request_hello_json(RRDHOST *host, struct web_client *w) {
     registry_json_header(host, w, "hello", REGISTRY_STATUS_OK);
 
-    const char *cloud_ui_url = appconfig_get(&cloud_config, CONFIG_SECTION_GLOBAL, "cloud ui url", DEFAULT_CLOUD_UI_URL);
+    if(host->node_id)
+        buffer_json_member_add_uuid(w->response.data, "node_id", host->node_id);
 
-    buffer_sprintf(w->response.data,
-            ",\n\t\"registry\": \"%s\",\n\t\"cloud_base_url\": \"%s\",\n\t\"anonymous_statistics\": %s",
-            registry.registry_to_announce,
-            cloud_ui_url, netdata_anonymous_statistics_enabled?"true":"false");
+    buffer_json_member_add_object(w->response.data, "agent");
+    {
+        buffer_json_member_add_string(w->response.data, "machine_guid", localhost->machine_guid);
+
+        if(localhost->node_id)
+            buffer_json_member_add_uuid(w->response.data, "node_id", localhost->node_id);
+
+        char *claim_id = get_agent_claimid();
+        if (claim_id) {
+            buffer_json_member_add_string(w->response.data, "claim_id", claim_id);
+            freez(claim_id);
+        }
+
+        buffer_json_member_add_boolean(w->response.data, "bearer_protection", netdata_is_protected_by_bearer);
+    }
+    buffer_json_object_close(w->response.data);
+
+    CLOUD_STATUS status = cloud_status();
+    buffer_json_member_add_string(w->response.data, "cloud_status", cloud_status_to_string(status));
+    buffer_json_member_add_string(w->response.data, "cloud_base_url", registry.cloud_base_url);
+
+    buffer_json_member_add_string(w->response.data, "registry", registry.registry_to_announce);
+    buffer_json_member_add_boolean(w->response.data, "anonymous_statistics", netdata_anonymous_statistics_enabled);
+    buffer_json_member_add_boolean(w->response.data, "X-Netdata-Auth", true);
+
+    buffer_json_member_add_array(w->response.data, "nodes");
+    RRDHOST *h;
+    dfe_start_read(rrdhost_root_index, h) {
+        buffer_json_add_array_item_object(w->response.data);
+        buffer_json_member_add_string(w->response.data, "machine_guid", h->machine_guid);
+
+        if(h->node_id)
+            buffer_json_member_add_uuid(w->response.data, "node_id", h->node_id);
+
+        buffer_json_member_add_string(w->response.data, "hostname", rrdhost_registry_hostname(h));
+        buffer_json_object_close(w->response.data);
+    }
+    dfe_done(h);
+    buffer_json_array_close(w->response.data); // nodes
 
     registry_json_footer(w);
-    return 200;
+    return HTTP_RESP_OK;
 }
 
 // ----------------------------------------------------------------------------
-//public ACCESS request
-
-#define REGISTRY_VERIFY_COOKIES_GUID "give-me-back-this-cookie-now--please"
+// public ACCESS request
 
 // the main method for registering an access
 int registry_request_access_json(RRDHOST *host, struct web_client *w, char *person_guid, char *machine_guid, char *url, char *name, time_t when) {
     if(unlikely(!registry.enabled))
         return registry_json_disabled(host, w, "access");
 
-    // ------------------------------------------------------------------------
-    // verify the browser supports cookies
-
-    if(registry.verify_cookies_redirects > 0 && !person_guid[0]) {
+    if(!registry_is_valid_url(url)) {
         buffer_flush(w->response.data);
-        registry_set_cookie(w, REGISTRY_VERIFY_COOKIES_GUID);
-        w->response.data->contenttype = CT_APPLICATION_JSON;
-        buffer_sprintf(w->response.data, "{ \"status\": \"redirect\", \"registry\": \"%s\" }", registry.registry_to_announce);
-        return 200;
+        buffer_strcat(w->response.data, "Invalid URL given in the request");
+        return HTTP_RESP_BAD_REQUEST;
     }
 
-    if(unlikely(person_guid[0] && !strcmp(person_guid, REGISTRY_VERIFY_COOKIES_GUID)))
+    // ------------------------------------------------------------------------
+    // verify the browser supports cookies or the bearer
+
+    if(registry.verify_cookies_redirects > 0 && !person_guid[0]) {
+        registry_lock();
+        registry_request_access(REGISTRY_VERIFY_COOKIES_GUID, machine_guid, url, name, when);
+        registry_unlock();
+
+        buffer_flush(w->response.data);
+        registry_set_cookie(w, REGISTRY_VERIFY_COOKIES_GUID);
+        w->response.data->content_type = CT_APPLICATION_JSON;
+        registry_json_header(host, w, "access", REGISTRY_STATUS_REDIRECT);
+        buffer_json_member_add_string(w->response.data, "person_guid", REGISTRY_VERIFY_COOKIES_GUID);
+        buffer_json_member_add_string(w->response.data, "registry", registry.registry_to_announce);
+        registry_json_footer(w);
+        return HTTP_RESP_OK;
+    }
+
+    if(unlikely(person_guid[0] && is_dummy_person(person_guid)))
+        // it passed the check - they gave us a different person_guid
+        // empty the dummy one, so that we will generate a new person_guid
         person_guid[0] = '\0';
 
     // ------------------------------------------------------------------------
@@ -208,7 +259,7 @@ int registry_request_access_json(RRDHOST *host, struct web_client *w, char *pers
         registry_json_header(host, w, "access", REGISTRY_STATUS_FAILED);
         registry_json_footer(w);
         registry_unlock();
-        return 412;
+        return HTTP_RESP_PRECOND_FAIL;
     }
 
     // set the cookie
@@ -216,15 +267,17 @@ int registry_request_access_json(RRDHOST *host, struct web_client *w, char *pers
 
     // generate the response
     registry_json_header(host, w, "access", REGISTRY_STATUS_OK);
+    buffer_json_member_add_string(w->response.data, "person_guid", p->guid);
+    buffer_json_member_add_array(w->response.data, "urls");
 
-    buffer_sprintf(w->response.data, ",\n\t\"person_guid\": \"%s\",\n\t\"urls\": [", p->guid);
     struct registry_json_walk_person_urls_callback c = { p, NULL, w, 0 };
-    avl_traverse(&p->person_urls, registry_json_person_url_callback, &c);
-    buffer_strcat(w->response.data, "\n\t]\n");
+    for(REGISTRY_PERSON_URL *pu = p->person_urls; pu ;pu = pu->next)
+        registry_json_person_url_callback(pu, &c);
+    buffer_json_array_close(w->response.data); // urls
 
     registry_json_footer(w);
     registry_unlock();
-    return 200;
+    return HTTP_RESP_OK;
 }
 
 // ----------------------------------------------------------------------------
@@ -235,6 +288,12 @@ int registry_request_delete_json(RRDHOST *host, struct web_client *w, char *pers
     if(!registry.enabled)
         return registry_json_disabled(host, w, "delete");
 
+    if(!registry_is_valid_url(url)) {
+        buffer_flush(w->response.data);
+        buffer_strcat(w->response.data, "Invalid URL given in the request");
+        return HTTP_RESP_BAD_REQUEST;
+    }
+
     registry_lock();
 
     REGISTRY_PERSON *p = registry_request_delete(person_guid, machine_guid, url, delete_url, when);
@@ -242,56 +301,77 @@ int registry_request_delete_json(RRDHOST *host, struct web_client *w, char *pers
         registry_json_header(host, w, "delete", REGISTRY_STATUS_FAILED);
         registry_json_footer(w);
         registry_unlock();
-        return 412;
+        return HTTP_RESP_PRECOND_FAIL;
     }
 
     // generate the response
     registry_json_header(host, w, "delete", REGISTRY_STATUS_OK);
     registry_json_footer(w);
     registry_unlock();
-    return 200;
+    return HTTP_RESP_OK;
 }
 
 // ----------------------------------------------------------------------------
 // public SEARCH request
 
 // the main method for searching the URLs of a netdata
-int registry_request_search_json(RRDHOST *host, struct web_client *w, char *person_guid, char *machine_guid, char *url, char *request_machine, time_t when) {
+int registry_request_search_json(RRDHOST *host, struct web_client *w, char *person_guid, char *request_machine) {
     if(!registry.enabled)
         return registry_json_disabled(host, w, "search");
 
+    if(!person_guid || !person_guid[0]) {
+        registry_json_header(host, w, "search", REGISTRY_STATUS_FAILED);
+        registry_json_footer(w);
+        return HTTP_RESP_PRECOND_FAIL;
+    }
+
     registry_lock();
 
-    REGISTRY_MACHINE *m = registry_request_machine(person_guid, machine_guid, url, request_machine, when);
+    STRING *hostname = NULL;
+    REGISTRY_MACHINE *m = registry_request_machine(person_guid, request_machine, &hostname);
     if(!m) {
         registry_json_header(host, w, "search", REGISTRY_STATUS_FAILED);
         registry_json_footer(w);
         registry_unlock();
-        return 404;
+        string_freez(hostname);
+        return HTTP_RESP_NOT_FOUND;
     }
 
     registry_json_header(host, w, "search", REGISTRY_STATUS_OK);
 
-    buffer_strcat(w->response.data, ",\n\t\"urls\": [");
+    buffer_json_member_add_array(w->response.data, "urls");
     struct registry_json_walk_person_urls_callback c = { NULL, m, w, 0 };
-    dictionary_walkthrough_read(m->machine_urls, registry_json_machine_url_callback, &c);
-    buffer_strcat(w->response.data, "\n\t]\n");
+
+    for(REGISTRY_MACHINE_URL *mu = m->machine_urls; mu ; mu = mu->next)
+        registry_json_machine_url_callback(mu, &c, hostname);
+
+    buffer_json_array_close(w->response.data);
 
     registry_json_footer(w);
     registry_unlock();
-    return 200;
+    string_freez(hostname);
+    return HTTP_RESP_OK;
 }
 
 // ----------------------------------------------------------------------------
 // SWITCH REQUEST
 
 // the main method for switching user identity
-int registry_request_switch_json(RRDHOST *host, struct web_client *w, char *person_guid, char *machine_guid, char *url, char *new_person_guid, time_t when) {
+int registry_request_switch_json(RRDHOST *host, struct web_client *w, char *person_guid, char *machine_guid, char *url __maybe_unused, char *new_person_guid, time_t when __maybe_unused) {
     if(!registry.enabled)
         return registry_json_disabled(host, w, "switch");
 
-    (void)url;
-    (void)when;
+    if(!person_guid || !person_guid[0]) {
+        buffer_flush(w->response.data);
+        buffer_strcat(w->response.data, "Who are you? Person GUID is missing");
+        return HTTP_RESP_PRECOND_FAIL;
+    }
+
+    if(!registry_is_valid_url(url)) {
+        buffer_flush(w->response.data);
+        buffer_strcat(w->response.data, "Invalid URL given in the request");
+        return HTTP_RESP_BAD_REQUEST;
+    }
 
     registry_lock();
 
@@ -322,7 +402,9 @@ int registry_request_switch_json(RRDHOST *host, struct web_client *w, char *pers
     struct registry_person_url_callback_verify_machine_exists_data data = { m, 0 };
 
     // verify the old person has access to this machine
-    avl_traverse(&op->person_urls, registry_person_url_callback_verify_machine_exists, &data);
+    for(REGISTRY_PERSON_URL *pu = op->person_urls; pu ;pu = pu->next)
+        registry_person_url_callback_verify_machine_exists(pu, &data);
+
     if(!data.count) {
         registry_json_header(host, w, "switch", REGISTRY_STATUS_FAILED);
         registry_json_footer(w);
@@ -332,7 +414,9 @@ int registry_request_switch_json(RRDHOST *host, struct web_client *w, char *pers
 
     // verify the new person has access to this machine
     data.count = 0;
-    avl_traverse(&np->person_urls, registry_person_url_callback_verify_machine_exists, &data);
+    for(REGISTRY_PERSON_URL *pu = np->person_urls; pu ;pu = pu->next)
+        registry_person_url_callback_verify_machine_exists(pu, &data);
+
     if(!data.count) {
         registry_json_header(host, w, "switch", REGISTRY_STATUS_FAILED);
         registry_json_footer(w);
@@ -346,11 +430,11 @@ int registry_request_switch_json(RRDHOST *host, struct web_client *w, char *pers
 
     // generate the response
     registry_json_header(host, w, "switch", REGISTRY_STATUS_OK);
-    buffer_sprintf(w->response.data, ",\n\t\"person_guid\": \"%s\"", np->guid);
+    buffer_json_member_add_string(w->response.data, "person_guid", np->guid);
     registry_json_footer(w);
 
     registry_unlock();
-    return 200;
+    return HTTP_RESP_OK;
 }
 
 // ----------------------------------------------------------------------------
@@ -380,7 +464,7 @@ void registry_statistics(void) {
         rrddim_add(sts, "sessions",  NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
     }
 
-    rrddim_set(sts, "sessions", registry.usages_count);
+    rrddim_set(sts, "sessions", (collected_number)registry.usages_count);
     rrdset_done(sts);
 
     // ------------------------------------------------------------------------
@@ -403,16 +487,14 @@ void registry_statistics(void) {
 
         rrddim_add(stc, "persons",        NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
         rrddim_add(stc, "machines",       NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
-        rrddim_add(stc, "urls",           NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
         rrddim_add(stc, "persons_urls",   NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
         rrddim_add(stc, "machines_urls",  NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
     }
 
-    rrddim_set(stc, "persons",       registry.persons_count);
-    rrddim_set(stc, "machines",      registry.machines_count);
-    rrddim_set(stc, "urls",          registry.urls_count);
-    rrddim_set(stc, "persons_urls",  registry.persons_urls_count);
-    rrddim_set(stc, "machines_urls", registry.machines_urls_count);
+    rrddim_set(stc, "persons",       (collected_number)registry.persons_count);
+    rrddim_set(stc, "machines",      (collected_number)registry.machines_count);
+    rrddim_set(stc, "persons_urls",  (collected_number)registry.persons_urls_count);
+    rrddim_set(stc, "machines_urls", (collected_number)registry.machines_urls_count);
     rrdset_done(stc);
 
     // ------------------------------------------------------------------------
@@ -435,15 +517,21 @@ void registry_statistics(void) {
 
         rrddim_add(stm, "persons",        NULL,  1, 1024, RRD_ALGORITHM_ABSOLUTE);
         rrddim_add(stm, "machines",       NULL,  1, 1024, RRD_ALGORITHM_ABSOLUTE);
-        rrddim_add(stm, "urls",           NULL,  1, 1024, RRD_ALGORITHM_ABSOLUTE);
         rrddim_add(stm, "persons_urls",   NULL,  1, 1024, RRD_ALGORITHM_ABSOLUTE);
         rrddim_add(stm, "machines_urls",  NULL,  1, 1024, RRD_ALGORITHM_ABSOLUTE);
     }
 
-    rrddim_set(stm, "persons",       registry.persons_memory + dictionary_stats_for_registry(registry.persons));
-    rrddim_set(stm, "machines",      registry.machines_memory + dictionary_stats_for_registry(registry.machines));
-    rrddim_set(stm, "urls",          registry.urls_memory);
-    rrddim_set(stm, "persons_urls",  registry.persons_urls_memory);
-    rrddim_set(stm, "machines_urls", registry.machines_urls_memory);
+    struct aral_statistics *p_aral_stats = aral_statistics(registry.persons_aral);
+    rrddim_set(stm, "persons",       (collected_number)p_aral_stats->structures.allocated_bytes + (collected_number)p_aral_stats->malloc.allocated_bytes + (collected_number)p_aral_stats->mmap.allocated_bytes);
+
+    struct aral_statistics *m_aral_stats = aral_statistics(registry.machines_aral);
+    rrddim_set(stm, "machines",      (collected_number)m_aral_stats->structures.allocated_bytes + (collected_number)m_aral_stats->malloc.allocated_bytes + (collected_number)m_aral_stats->mmap.allocated_bytes);
+
+    struct aral_statistics *pu_aral_stats = aral_statistics(registry.person_urls_aral);
+    rrddim_set(stm, "persons_urls",  (collected_number)pu_aral_stats->structures.allocated_bytes + (collected_number)pu_aral_stats->malloc.allocated_bytes + (collected_number)pu_aral_stats->mmap.allocated_bytes);
+
+    struct aral_statistics *mu_aral_stats = aral_statistics(registry.machine_urls_aral);
+    rrddim_set(stm, "machines_urls", (collected_number)mu_aral_stats->structures.allocated_bytes + (collected_number)mu_aral_stats->malloc.allocated_bytes + (collected_number)mu_aral_stats->mmap.allocated_bytes);
+
     rrdset_done(stm);
 }
