@@ -14,7 +14,7 @@
 void free_silencers(SILENCER *t) {
     if (!t) return;
     if (t->next) free_silencers(t->next);
-    debug(D_HEALTH, "HEALTH command API: Freeing silencer %s:%s:%s:%s:%s", t->alarms,
+    netdata_log_debug(D_HEALTH, "HEALTH command API: Freeing silencer %s:%s:%s:%s:%s", t->alarms,
           t->charts, t->contexts, t->hosts, t->families);
     simple_pattern_free(t->alarms_pattern);
     simple_pattern_free(t->charts_pattern);
@@ -96,12 +96,12 @@ void health_silencers2file(BUFFER *wb) {
     if(fd) {
         size_t written = (size_t)fprintf(fd, "%s", wb->buffer) ;
         if (written == wb->len ) {
-            info("Silencer changes written to %s", silencers_filename);
+            netdata_log_info("Silencer changes written to %s", silencers_filename);
         }
         fclose(fd);
         return;
     }
-    error("Silencer changes could not be written to %s. Error %s", silencers_filename, strerror(errno));
+    netdata_log_error("Silencer changes could not be written to %s. Error %s", silencers_filename, strerror(errno));
 }
 
 /**
@@ -121,7 +121,7 @@ int web_client_api_request_v1_mgmt_health(RRDHOST *host, struct web_client *w, c
 
     BUFFER *wb = w->response.data;
     buffer_flush(wb);
-    wb->contenttype = CT_TEXT_PLAIN;
+    wb->content_type = CT_TEXT_PLAIN;
 
     buffer_flush(w->response.data);
 
@@ -133,20 +133,20 @@ int web_client_api_request_v1_mgmt_health(RRDHOST *host, struct web_client *w, c
         buffer_strcat(wb, HEALTH_CMDAPI_MSG_AUTHERROR);
         ret = HTTP_RESP_FORBIDDEN;
     } else {
-        debug(D_HEALTH, "HEALTH command API: Comparing secret '%s' to '%s'", w->auth_bearer_token, api_secret);
+        netdata_log_debug(D_HEALTH, "HEALTH command API: Comparing secret '%s' to '%s'", w->auth_bearer_token, api_secret);
         if (strcmp(w->auth_bearer_token, api_secret)) {
             buffer_strcat(wb, HEALTH_CMDAPI_MSG_AUTHERROR);
             ret = HTTP_RESP_FORBIDDEN;
         } else {
             while (url) {
-                char *value = mystrsep(&url, "&");
+                char *value = strsep_skip_consecutive_separators(&url, "&");
                 if (!value || !*value) continue;
 
-                char *key = mystrsep(&value, "=");
+                char *key = strsep_skip_consecutive_separators(&value, "=");
                 if (!key || !*key) continue;
                 if (!value || !*value) continue;
 
-                debug(D_WEB_CLIENT, "%llu: API v1 health query param '%s' with value '%s'", w->id, key, value);
+                netdata_log_debug(D_WEB_CLIENT, "%llu: API v1 health query param '%s' with value '%s'", w->id, key, value);
 
                 // name and value are now the parameters
                 if (!strcmp(key, "cmd")) {
@@ -171,7 +171,7 @@ int web_client_api_request_v1_mgmt_health(RRDHOST *host, struct web_client *w, c
                         silencers->silencers = NULL;
                         buffer_strcat(wb, HEALTH_CMDAPI_MSG_RESET);
                     } else if (!strcmp(value, HEALTH_CMDAPI_CMD_LIST)) {
-                        w->response.data->contenttype = CT_APPLICATION_JSON;
+                        w->response.data->content_type = CT_APPLICATION_JSON;
                         health_silencers2json(wb);
                         config_changed=0;
                     }
@@ -196,7 +196,7 @@ int web_client_api_request_v1_mgmt_health(RRDHOST *host, struct web_client *w, c
     w->response.data = wb;
     buffer_no_cacheable(w->response.data);
     if (ret == HTTP_RESP_OK && config_changed) {
-        BUFFER *jsonb = buffer_create(200);
+        BUFFER *jsonb = buffer_create(200, &netdata_buffers_statistics.buffers_health);
         health_silencers2json(jsonb);
         health_silencers2file(jsonb);
         buffer_free(jsonb);
